use crate::config::Context;

use serde_json;
use std::io;
use std::sync::Arc;
use tokio::io::AsyncWriteExt;
use tokio::net::TcpSocket;
use tokio::sync::RwLock;

pub async fn apiserver(ctxarc: &Arc<RwLock<Context>>) -> io::Result<()> {
    let ctxarc = ctxarc.clone();
    let addr = { ctxarc.read().await.cfg.api.parse().unwrap() };

    let socket = TcpSocket::new_v4().unwrap();
    match socket.bind(addr) {
        Ok(_) => {}
        Err(_) => {
            println!("can't bind monitoring socket, exiting...");
            std::process::exit(1);
        }
    }
    socket.set_reuseaddr(true).unwrap();
    socket
        .set_linger(Some(std::time::Duration::from_secs(0)))
        .unwrap();
    let listener = socket.listen(128).unwrap();

    tokio::spawn(async move {
        loop {
            match listener.accept().await {
                Ok((stream, _addr)) => {
                    stream.readable().await.unwrap();
                    let (reader, mut writer) = stream.into_split();
                    let mut buf: [u8; 16] = [0; 16];

                    match reader.try_read(&mut buf) {
                        Ok(_) => {}
                        Err(e) => {
                            println!("error: {}", e);
                        }
                    };
                    let msg = match String::from_utf8(buf.to_vec()) {
                        Ok(o) => o.trim_matches(char::from(0)).trim().to_string(),
                        Err(_) => "".to_string(),
                    };

                    let res = format_result(&ctxarc, msg.as_str()).await;

                    match writer.write_all(format!("{res}").as_bytes()).await {
                        Ok(_) => {}
                        Err(err) => {
                            println!("ee {err}");
                        }
                    }
                }
                Err(err) => {
                    println!("unable to serialize data: {err}");
                }
            }
        }
    });
    Ok(())
}

async fn format_result(ctxarc: &Arc<RwLock<Context>>, mode: &str) -> String {
    let data;
    let ctx = ctxarc.read().await;
    match mode {
        "cfg" => data = serde_json::to_string(&ctx.cfg).unwrap(),
        "blocklist" => data = serde_json::to_string(&ctx.blocklist).unwrap(),
        _ => data = serde_json::to_string(&ctx.blocklist).unwrap(),
    };
    data
}
